'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = bindImports;

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function bindImports(ast) {
  let container;
  let dependencies = [];

  (0, _babelTraverse2.default)(ast, {
    Program(path) {
      container = path.scope.generateUidIdentifier('modules');
    },
    VariableDeclarator(path) {
      const init = path.node.init;
      const identifier = path.node.id;
      const isCallExpression = t.isCallExpression(init);

      const isRequireCall = isCallExpression && init.callee.name === 'require';

      const isStaticRequireCall = isRequireCall && init.arguments.length === 1 && t.isStringLiteral(init.arguments[0]);

      // like: var foo = require('./bar');
      if (isStaticRequireCall) {
        const requirePath = init.arguments[0].value;
        // Only work on local requires
        if (requirePath[0] !== '.') {
          return;
        }

        dependencies.push({
          name: identifier.name,
          path: requirePath,
          init: init
        });

        path.remove();
        path.scope.rename(identifier.name, `${ container.name }.${ identifier.name }`);
      }
    }
  });

  (0, _babelTraverse2.default)(ast, {
    Program(path) {
      const properties = dependencies.map(dependency => {
        return t.objectMethod('get', t.identifier(dependency.name), [], t.BlockStatement([t.returnStatement(dependency.init)]));
      });
      if (properties.length > 0) {
        const modules = t.objectExpression(properties);
        const modulesDeclaration = t.VariableDeclaration('const', [t.VariableDeclarator(container, modules)]);
        path.node.body.unshift(modulesDeclaration);
      }
    }
  });

  return ast;
}